/*
Copyright 2002 - 2004 UnitedAdmins
Project originally created by OLO (http://olo.counter-strike.pl)

This file is part of StatsMe.

   StatsMe is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   StatsMe is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with StatsMe; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   A copy of the GNU General Public License is available in a file called COPYING.
   See README for more information.

*/

/* $Id: statsme.cpp,v 1.34 2004/04/05 11:58:34 punisher Exp $ */

#include <extdll.h>			// always

#include <api_info.h>		// api_info_t, etc
#include <sdk_util.h>		// REG_SVR_COMMAND, etc
#include <meta_api.h>		// Plugin_info, etc
#include "statsme.h"	// me
#include <ctype.h>

char* UTIL_Itoa(int i) {
	static char a[32];
	sprintf(a,"%d",i);
	return a;
}

char* UTIL_VarArgs( char *fmt, ... ){
	va_list	argptr;
	static char	string[2048];
	va_start (argptr, fmt);
	vsprintf (string, fmt,argptr);
	va_end (argptr);
	return string;
}

// converts: "abc" to bit sum: 1|2|4
int UTIL_ReadFlags(const char* c) {
	int flags = atoi( c );
	if ( !flags )
		while (*c) 
			flags |= ( 1 << ( *c++ - 'a' ) );
	return flags;
}


char* UTIL_RemoveQuotes(char* arg) {
	if (*arg=='\"') {// remove quotes
		int len = strlen(arg);
		if (arg[len-1]=='\r'&&arg[len-2]=='\"') {// exception for some win32 editors
			arg[len-2] = 0;
			arg++;
		}
		else if (arg[len-1]=='\"')	{
			arg[len-1] = 0;
			arg++;
		}
	}
	return arg;
}

// checks if string represents the number
bool UTIL_IsDigit(const char* value) {
	if ( !*value ) 
		return false;
	if ( *value == '-' )
		++value; // allow negative at begining
	bool dot = true;
	while ( *value ) {
		if ( *value>'9' || *value<'0' )	{
			if ( *value=='.' && dot && *(value+1) ) {
				dot = false; //accept only one dot
				++value;
				continue;
			}
			return false;
		}
		++value;
	}
	return true;
}

const char* UTIL_Pathname( char *fmt, ... ) {
	va_list	argptr;
	static char	string[256];
	va_start (argptr, fmt);
	vsprintf (string, fmt,argptr);
	va_end (argptr);
	char* path = string;
	while (*path) { 
#ifndef __linux__
		if (*path == '/') *path = '\\'; // convert to MS-DOS format 
#else
		if (*path == '\\') *path = '/'; // convert to Linux format
#endif
		++path;
	}
	return string;
}

player_t* UTIL_GetPlayerByName(const char* name)
{	
	for(int i = 1; i <= gpGlobals->maxClients; ++i) {
		player_t* pPlayer = GET_PLAYER_POINTER_I(i);
		if ( strcmp(pPlayer->name, name) == 0 )
			return pPlayer;
	}

	return NULL;
}

int UTIL_GetPlayersNumber()
{
	int num = 0;
	for(int i = 1; i <= gpGlobals->maxClients; ++i) {
		player_t*pPlayer = GET_PLAYER_POINTER_I(i);
		if (pPlayer->ingame) 
			num++;
	}
	return num;
}

// find out what MOD we are running
// in case of failure the name will be "valve" and id will be "0"
/*void UTIL_GetModName(char* name, int* id) {

	static struct mod_info_t {
		const char* name; 
		int id;
	} mods[] = {
		{"cstrike", 1 },
		{"dod", 2 },
		{"tfc", 3 },
		{"valve", 0 }
	};

	char game_dir[256];
	GET_GAME_DIR(game_dir);

	char *mod_name = game_dir;
	int i = 0;

	while ( game_dir[i] ) // catch word after last '/'
		if (game_dir[i++] == '/') 
			mod_name = &game_dir[i];

	for (i = 0; mods[i].id; ++i) // look for name in struct above
		if ( !strcmpi( mod_name, mods[i].name ) )	
			break;

	*id = mods[i].id;
	strcpy(name,mods[i].name);
} */

void UTIL_ClientPrint( edict_t *pEntity, int msg_dest,  char *msg )
{
	if ( gmsgTextMsg == 0) {
		LOG_MESSAGE(PLID, "ERROR: (textmsg) no type for message.");
		return;
	}

	if ( strlen( msg ) > 189 )
		msg[190] = 0;

	MESSAGE_BEGIN( MSG_ONE, gmsgTextMsg, NULL, pEntity );
		WRITE_BYTE( msg_dest );
		WRITE_STRING( msg );
	MESSAGE_END();
}


// used to remove some menu tags like: \R, \y, \w and \d
void UTIL_ClearMenuTags(register char *b)
{
	register char*a = b;
	while(*b) {
		if (*b == '\\') {
			int c = *(b+1);
			if (c=='R')	{ // white space
				*a++ = ' ';
				b += 2;
				continue;
			}
			else if (c=='w'||c=='d'||c=='y') { // info tags
				b += 2;
				continue;
			}
		}
		*a++ = *b++;
	}
	*a = 0;
}

void UTIL_ShowMenu( edict_t* pEdict, int slots, int time, char *menu )
{
	if ( gmsgShowMenu == 0) {
		LOG_MESSAGE(PLID, "ERROR: (showmenu) no type for message.");
		return;
	}

	if (sm.mod_id != MOD_CSTRIKE)
		UTIL_ClearMenuTags( menu );

	char *n = menu;
	int	c = 0;

	while ( *n ) {
		int a = strlen( n );
		if ( a > 174 ) a = 175;
		c = *(n+=a);
		*n = '\0';
		MESSAGE_BEGIN( MSG_ONE, gmsgShowMenu, NULL, pEdict );	
			WRITE_SHORT( slots );
			WRITE_CHAR( time );
			WRITE_BYTE( c ? TRUE : FALSE); //Need more menu?
			WRITE_STRING( menu );
		MESSAGE_END();
		*n = c;
		menu = n;
	}

}

void UTIL_ShowMOTD( edict_t *client , char *motd, const char *name)
{
	if ( gmsgServerName == 0) {
		LOG_MESSAGE(PLID, "ERROR: (servername) no type for message.");
		return;
	}
	
	if ( gmsgMOTD == 0) {
		LOG_MESSAGE(PLID, "ERROR: (motd) no type for message.");
		return;
	}

	MESSAGE_BEGIN( MSG_ONE, gmsgServerName, NULL, client );
		WRITE_STRING(name);
	MESSAGE_END();

	char *n = motd;
	// Add HTML parts to the MOTD ( CS 1.6 / DoD Steam )
	char *htmlmotd = UTIL_VarArgs("<html><head><title>%s</title></head><body style=\"font-family: %s; color: %s;\" bgcolor=\"%s\"><table border=0 width=100%%>%s</table></body></html>",name,sm_htmlfont->string,sm_htmlfgcolor->string,sm_htmlbgcolor->string,motd);
	int	c = 0;
	if (sm.mod_set.vgui2 && sm_htmlvgui2->value)
		n = htmlmotd;
	while ( *n )
	{
		int a = strlen( n );
		
		if ( a > 59 ) a = 60;
		c = *(n+=a);
		*n = '\0';
		MESSAGE_BEGIN( MSG_ONE, gmsgMOTD, NULL, client );
			WRITE_BYTE( c ? FALSE : TRUE );
		if (sm.mod_set.vgui2 && sm_htmlvgui2->value) {
				WRITE_STRING( htmlmotd );
			MESSAGE_END();
			*n = c;
			htmlmotd = n;
		} else {
				WRITE_STRING( motd );
			MESSAGE_END();
			*n = c;
			motd = n;
		}
	}
	MESSAGE_BEGIN( MSG_ONE, gmsgServerName, NULL, client );
		WRITE_STRING(CVAR_GET_STRING("hostname"));
	MESSAGE_END();
}

char* UTIL_SplitHudMessage(register const char *src)
{
	static char message[492];
	register int a = 0; // num of chars from last space
	register int b = 0; // new line length
	register int d = 0; // source position
	register int e = 0; // overall egth
	int c = -1; // position of last space
	// go through text and move to new line if needed
	while ( e < 490 && src[ d ] ) {
		if ( src[ d ] == ' ' && b < 72 ) {
			c = d;
			a = 0;
		}
		if ( src[d] == '\n' ) {
			c = -1;
			b = 0;
		}
		message[ e++ ] = src[ d++ ];
		a++;
		if ( ++b > 71 )	{
			if ( c == -1 ) {// no space in word so we must add nextline
				message[ e++ ] = '\n';
				b = 0;
			}
			else {
				message[ c ] = '\n';
				b = a;
				c = -1;
			}
		}
	}
	message[ e ] = 0;
	return message;
}

// converts integer to string representation (range from 1 to 9999)
void UTIL_IntToString(int value, char *output)
{
	static const char *words[] = {"zero","one ","two ","three ","four ",
		"five ", "six ","seven ","eight ","nine ","ten ", //10
		"eleven ","twelve ","thirteen ","fourteen ","fifteen ",
		"sixteen ","seventeen ","eighteen ","nineteen ", //19
		"twenty ","thirty ","fourty ", "fifty ","sixty ",
		"seventy ","eighty ","ninety ",//27
		"hundred ","thousand "};
	*output = 0;
	if (value < 0) value = -value; // flip negative value
	int tho = value / 1000;
	if (tho){
		strcat(output, words[ tho ] );	
		strcat(output, words[29] );	
		value =  value % 1000;
	}
	int hun = value / 100;
	if (hun) {
		strcat(output, words[ hun ] );	
		strcat(output, words[28] );	
		value =  value % 100;
	}
	int ten = value / 10;
	int	unit = value % 10;
	if ( ten )	strcat(output, words[ ( ten > 1 ) ? ( ten + 18 ) : ( unit + 10 ) ] );
	if ( ten != 1 && ( unit || (!value && !hun && !tho) ) ) strcat(output, words[ unit ] );
}

void UTIL_GetSystemTime(char* dest, unsigned int size, const char* format)
{
	time_t td; time(&td);
	strftime(dest,size,format,localtime(&td));
}

char* UTIL_PutSpecChars(register const char* b)
{
	static char message[512];
	register char*a = message;
	while(*b) {
		if ( *b == '\\' ) { // replace \' with " and \n with nl
			switch ( *(b+1) ) {
			case '\'':
				*a++ = '\"';
				b+=2;
				continue;
			case 'n':
				*a++ = '\n';
				b+=2;
				continue;
			case '\\':
				*a++ = '\\';
				b+=2;
				continue;
			}
		}
		*a++ = *b++;
	}
	*a++ = '\n';
	*a = 0;
	return message;
}

void SettingsMenu(player_t *pPlayer, int state, action_t* a)
{
	if (state < 0){ //we have pressed exit
		pPlayer->menu_type = MENU_IDLE;
		return;
	}
	char textmenu[512]; int list_end, list_begin, pages_num = 0, pos_num = 0, index_key = 0, keys = 0;
	for(action_t*s = a;s;s=s->next) ++pos_num;
	pages_num = (pos_num / 8) + ((pos_num % 8)?1:0);
	int len=sprintf(textmenu, "\\yStatsMe Menu \\R%d/%d\n\\w\n", state + 1, pages_num);
	pPlayer->menu_state = state; //set position in menu (which page)
	list_begin = (state*8); //get begin of list
	list_end = (pos_num > list_begin + 8) ? (list_begin + 8) : pos_num;
	for(int c = 0;c<list_begin;c++)	a=a->next;
	for (int i=list_begin;i<list_end;++i) {
		if (a->cmd && a->cmd[0] == ' ' && !a->cmd[1]){
			len+=sprintf(&textmenu[len], (char*)"\\d    %s\\w",smv_putvars(a->name) );
			++index_key;
		}
		else {
			len+=sprintf(&textmenu[len], (char*)"\\w%d.  %s", index_key+1, smv_putvars(a->name) );
			keys += (1<<index_key++); //Add proper key to menu keys
		}
		a = a->next;
	}
	if (list_end < pos_num){
		len+=sprintf(&textmenu[len], "\n\\w9.  More...");
		keys += (1<<8);
	}
	strcat(&textmenu[len], state ? "\n\\w0.  Back" : "\n\\w0.  Exit");
	keys+=(1<<9);
	UTIL_ShowMenu(pPlayer->pEdict, keys, -1,textmenu);
	pPlayer->menu_type = SETTINGS_MENU;
	pPlayer->menu_keys = keys;
}

void StatsMenu(player_t *pPlayer, int state)
{
	if (state < 0){ //we have pressed exit
		pPlayer->menu_type = MENU_IDLE;
		return;
	}
	pPlayer->menu_pplnum = 0; //get players list
	for (int a = 1; a <= gpGlobals->maxClients; ++a)
		if (players[a].ingame)	pPlayer->menu_list[pPlayer->menu_pplnum++] = &players[a];
	int list_end, list_begin, pages_num = 0, index_key = 0, keys = 0; char textmenu[512];
	pages_num = (pPlayer->menu_pplnum / 8) + ((pPlayer->menu_pplnum % 8)?1:0); //get number of all pages
	int len = sprintf(textmenu, "\\yStats Menu \\R%d/%d\n\\w\n" , state + 1, pages_num); //Print header
	pPlayer->menu_state = state; //set position in menu (which page)
	list_begin = (state*8); //get begin of list
	//check if this is a last page or we are at the middle
	list_end = (pPlayer->menu_pplnum > list_begin + 8) ? (list_begin + 8) : pPlayer->menu_pplnum;
	for (int i=list_begin;i<list_end;++i) {
		len+=sprintf(&textmenu[len], "%d.  %s\n", index_key+1, STRING(pPlayer->menu_list[i]->pEdict->v.netname));
		keys += (1<<index_key++); //Add proper key to menu keys
	}
	if (list_end < pPlayer->menu_pplnum){
		len+=sprintf(&textmenu[len],"\n9.  More...");
		keys += (1<<8);
	}
	strcat(&textmenu[len], state?"\n0.  Back":"\n0.  Exit");
	keys+=(1<<9);
	UTIL_ShowMenu(pPlayer->pEdict, keys, -1,textmenu);
	pPlayer->menu_type = STATS_MENU;
	pPlayer->menu_keys = keys;
}

void ResetStats( player_t* pPlayer )
{
	memset(pPlayer->aWpn_stats ,0,sizeof(stat_t)*MAX_WEAPONS);
	memset(&pPlayer->aGm_stats,0,sizeof(stat_t));
	memset(&pPlayer->aRnd_stats,0,sizeof(stat_t));
	memset(pPlayer->aPpl_stats,0,sizeof(stat_t)*33);
	memset(pPlayer->aAPpl_stats,0,sizeof(stat_t)*33);
	memset(&pPlayer->aMon_stats,0,sizeof(stat_t));
}

void DeleteConfiguration()
{
	for(int i = 0; i < MAX_SM_REG_MSGS ; ++i){
		if (!modMsgsFunCall[i])	continue;
		while (modMsgsFunCall[i]){
			fun_call_t*fc_todel = modMsgsFunCall[i];
			modMsgsFunCall[i] = modMsgsFunCall[i]->next;
			for(int b=0;b<MAX_PARSE_VALUES;b++){
				while(fc_todel->cond[b]) {//delete all conditions from fun_call_t
					cond_t*cnd_todel = fc_todel->cond[b];
					fc_todel->cond[b] = fc_todel->cond[b]->next;
					if (cnd_todel->sValue) delete[] cnd_todel->sValue;
					delete cnd_todel;
				}
			}
			delete[] fc_todel->cmd;
			delete fc_todel;
		}
	}
	
	while(sm_motd) { 	         
		motd_t*motd_todel = sm_motd; 	                 
		sm_motd = sm_motd->next; 	 
		delete[] motd_todel->msg; 	 
		delete motd_todel; 	 
	}
	
	while(sm_menu) {
		action_t*menu_todel = sm_menu;
		sm_menu = sm_menu->next;
		delete[] menu_todel->name;
		delete[] menu_todel->cmd;
		delete menu_todel;
	}
	
	while(sm_alias) {
		action_t*alias_todel = sm_alias;
		sm_alias = sm_alias->next;
		delete[] alias_todel->name;
		delete[] alias_todel->cmd;
		delete alias_todel;
	}
				
	while(sm_var) {
		myvar_t*myvar_todel = sm_var;
		sm_var = sm_var->next;
		delete[] myvar_todel->name;
		delete[] myvar_todel->string;
		delete myvar_todel;
	}

	while(sm_task) {
		task_t*task_todel = sm_task;
		sm_task = sm_task->next;
		delete[] task_todel->cmd;
		delete task_todel;
	}

	while (sm_grenades){
		grenades_t*to_del = sm_grenades;
		sm_grenades = sm_grenades->next;
		delete to_del;
	}

	while (sm_engtask){
		engtask_t*to_del = sm_engtask;
		sm_engtask = sm_engtask->next;
		delete[] to_del->cmd;
		delete to_del;
	}
}

void PutGrenade(edict_t* pGre, float time,int type, player_t* pPlayer)
{
	grenades_t* newone = new grenades_t;
	newone->time = gpGlobals->time + time;
	newone->pPlayer = pPlayer;
	newone->pGrenade = pGre;
	newone->index = type;
	newone->prev = NULL;
	newone->next = sm_grenades;
	if (sm_grenades) sm_grenades->prev = newone;
	sm_grenades = newone;
}

grenades_t* FindGrenade(edict_t* enemy)
{
	grenades_t* result = NULL;

	for(grenades_t* p = sm_grenades; p ; ) {
		if ( p->time > gpGlobals->time ) {
			if (p->pGrenade == enemy){
				result = p; //why not just return? - coz we must check for outdated grenades
			}
		}
		else{
			// if the grenade is outdated then delete it from list
			grenades_t* next = p->next;
			if (p->prev)
				p->prev->next = next;
			else
				sm_grenades = next;
			if (next)
				next->prev = p->prev;
			delete p;
			p = next;
			continue;
		}
		p = p->next;
	}

	return result;
}

void FireShot(player_t* pPlayer, int weapon)
{
	pPlayer->pWpn_stats[weapon].shots++;
	//pPlayer->pPpl_stats[???].shots++;
	pPlayer->pRnd_stats->shots++;
	pPlayer->pGm_stats->shots++;
	pPlayer->pMon_stats->shots++;
	
	sm.gamestats[pPlayer->team_index].shots++;
	sm.roundstats[pPlayer->team_index].shots++;
	sm.all_roundstats.shots++;
	sm.all_gamestats.shots++;
}

inline void CHECK_FOR_BEST(player_t** cur, player_t*pl,int v1, int v2, int o1, int o2) {
	if (v1 > o1 || ( v1 == o1 && v2 > o2) )	*cur = pl;
}

void FireHit(player_t* pVictim,player_t* pAttacker, int weapon, int damage, int aimed_at)
{	
	pAttacker->pWpn_stats[weapon].damage += damage;
	pAttacker->pWpn_stats[weapon].hits++;
	pAttacker->pWpn_stats[weapon].hitgroup[aimed_at]++;
			
	pAttacker->pPpl_stats[pVictim->index].damage += damage;
	pAttacker->pPpl_stats[pVictim->index].hits++;
	pAttacker->pPpl_stats[pVictim->index].hitgroup[aimed_at]++;

	pVictim->pAPpl_stats[pAttacker->index].damage += damage;
	pVictim->pAPpl_stats[pAttacker->index].hits++;
	pVictim->pAPpl_stats[pAttacker->index].hitgroup[aimed_at]++;

	pAttacker->pRnd_stats->damage += damage;
	pAttacker->pRnd_stats->hits++;
	pAttacker->pRnd_stats->hitgroup[aimed_at]++;
	
	pAttacker->pGm_stats->damage += damage;
	pAttacker->pGm_stats->hits++;
	pAttacker->pGm_stats->hitgroup[aimed_at]++;

	pAttacker->pMon_stats->damage += damage;
	pAttacker->pMon_stats->hits++;
	pAttacker->pMon_stats->hitgroup[aimed_at]++;
	
	sm.gamestats[pAttacker->team_index].damage += damage;
	sm.gamestats[pAttacker->team_index].hits++;
	sm.gamestats[pAttacker->team_index].hitgroup[aimed_at]++;
	
	sm.all_roundstats.damage += damage;
	sm.all_roundstats.hits++;
	sm.all_roundstats.hitgroup[aimed_at]++;
	
	sm.roundstats[pAttacker->team_index].damage += damage;
	sm.roundstats[pAttacker->team_index].hits++;
	sm.roundstats[pAttacker->team_index].hitgroup[aimed_at]++;
	
	sm.all_gamestats.damage += damage;
	sm.all_gamestats.hits++;
	sm.all_gamestats.hitgroup[aimed_at]++;

	CHECK_FOR_BEST(&sm.gm_disruptive, pAttacker ,
		pAttacker->pGm_stats->damage, 
		pAttacker->pGm_stats->hits, 
		sm.gm_disruptive->pGm_stats->damage,
		sm.gm_disruptive->pGm_stats->hits);

	CHECK_FOR_BEST(&sm.rnd_disruptive, pAttacker ,
		pAttacker->pRnd_stats->damage, 
		pAttacker->pRnd_stats->hits, 
		sm.rnd_disruptive->pRnd_stats->damage,
		sm.rnd_disruptive->pRnd_stats->hits);

	CHECK_FOR_BEST(&sm.gm_accurate, pAttacker ,
		pAttacker->pGm_stats->hitgroup[HITGROUP_HEAD], 
		pAttacker->pGm_stats->hitgroup[HITGROUP_STOMACH] + 
		pAttacker->pGm_stats->hitgroup[HITGROUP_CHEST], 
		sm.gm_accurate->pGm_stats->hitgroup[HITGROUP_HEAD],
		sm.gm_accurate->pGm_stats->hitgroup[HITGROUP_STOMACH] + 
		sm.gm_accurate->pGm_stats->hitgroup[HITGROUP_CHEST]);

	CHECK_FOR_BEST(&sm.rnd_accurate, pAttacker ,
		pAttacker->pRnd_stats->hitgroup[HITGROUP_HEAD], 
		pAttacker->pRnd_stats->hitgroup[HITGROUP_STOMACH] + 
		pAttacker->pRnd_stats->hitgroup[HITGROUP_CHEST], 
		sm.rnd_accurate->pRnd_stats->hitgroup[HITGROUP_HEAD],
		sm.rnd_accurate->pRnd_stats->hitgroup[HITGROUP_STOMACH] + 
		sm.rnd_accurate->pRnd_stats->hitgroup[HITGROUP_CHEST]);

}

void FireKill(player_t* pVictim, player_t* pKiller, int weapon, int hs, bool tk)
{
	if ( hs ) { //headshot
		pKiller->pGm_stats->hs++; // game normal score
		pKiller->pRnd_stats->hs++; // round normal score
		pKiller->pWpn_stats[weapon].hs++; //save in weapon stats
		pKiller->pPpl_stats[pVictim->index].hs++; //in round detailed stats
		pVictim->pAPpl_stats[pKiller->index].hs++; //in round detailed stats		
		sm.gamestats[pKiller->team_index].hs++; //game team score
		sm.roundstats[pKiller->team_index].hs++; //round team score
		sm.all_roundstats.hs++;
		sm.all_gamestats.hs++;
	}

	float distance = (pVictim->pEdict->v.origin - pKiller->pEdict->v.origin).Length();
			
	pKiller->pWpn_stats[weapon].kills++;
	pKiller->pPpl_stats[pVictim->index].kills++;
	pVictim->pAPpl_stats[pKiller->index].kills++;
	pKiller->pRnd_stats->kills++;
	pKiller->pGm_stats->kills++;
	pKiller->pPpl_stats[pVictim->index].dist = distance;
	pVictim->pAPpl_stats[pKiller->index].dist = distance;	
	sm.gamestats[pKiller->team_index].kills++;
	sm.roundstats[pKiller->team_index].kills++;
	
	sm.all_roundstats.kills++;
	sm.all_gamestats.kills++;
	
	pVictim->pWpn_stats[weapon].deaths++;
	pVictim->pPpl_stats[pVictim->index].deaths++; //round detailed
		
	pVictim->pRnd_stats->deaths++;
	pVictim->pGm_stats->deaths++;
	pVictim->pPpl_stats[pVictim->index].dist = distance;
	
	sm.gamestats[pVictim->team_index].deaths++;
	sm.roundstats[pVictim->team_index].deaths++;
	
	sm.all_roundstats.deaths++;
	sm.all_gamestats.deaths++;
	
	if (tk) {//team killing
		pKiller->pWpn_stats[weapon].tks++;
		pKiller->pPpl_stats[pVictim->index].tks++; //round detailed
		pVictim->pAPpl_stats[pKiller->index].tks++; //round detailed
		pKiller->pRnd_stats->tks++;
		pKiller->pGm_stats->tks++;
		sm.gamestats[pKiller->team_index].tks++;
		sm.roundstats[pKiller->team_index].tks++;
		sm.all_roundstats.tks++;
		sm.all_gamestats.tks++;
	}
	else {				
		pKiller->monster_time = gpGlobals->time + sm_monsterfreq->value;
		pKiller->pMon_stats->kills++;

		if (hs)	{
			pKiller->pMon_stats->hs++;
		}
	}

	CHECK_FOR_BEST(&sm.gm_killer, pKiller ,
		pKiller->pGm_stats->kills, 
		pKiller->pGm_stats->hs, 
		sm.gm_killer->pGm_stats->kills,
		sm.gm_killer->pGm_stats->hs);

	CHECK_FOR_BEST(&sm.rnd_killer, pKiller ,
		pKiller->pRnd_stats->kills, 
		pKiller->pRnd_stats->hs, 
		sm.rnd_killer->pRnd_stats->kills,
		sm.rnd_killer->pRnd_stats->hs);
}

char* GetVictimsList(player_t *pPlayer)
{
	sm.player = pPlayer;
	player_t* pVictim = pPlayer->victim;
	stat_t* pCustom = pPlayer->custom;
	*szBuffor = 0;
	int len = 0;
	for (int a = 1; a <= gpGlobals->maxClients; ++a)	
		if ( pPlayer->pPpl_stats[a].hits && pPlayer->index != a  ){
			player_t* pVictim = GET_PLAYER_POINTER_I(a);
			pPlayer->victim = pVictim;
			pPlayer->custom = &pPlayer->pPpl_stats[a];
			stat_t* vCustom = pVictim->custom;
			pVictim->custom = &pVictim->pPpl_stats[pPlayer->index];
			len += sprintf( &szBuffor[len], smv_putvars( sm_victims->string )  );	
			if ( len ) {  // convert '\n' to "\n"
				szBuffor[len-1] = '\\';
				szBuffor[len] = 'n';
				szBuffor[++len] = 0;
			}
			pVictim->custom = vCustom;
		}
	pPlayer->victim = pVictim;
	pPlayer->custom = pCustom;
	return szBuffor;
}


char* GetAttackersList(player_t *pPlayer)
{
	sm.player = pPlayer;
	player_t* pAttacker = pPlayer->killer;
	stat_t* pCustom = pPlayer->custom;
	*szBuffor = 0;
	int len = 0;
	for (int a = 1; a <= gpGlobals->maxClients; ++a)	
		if ( pPlayer->pAPpl_stats[a].hits && pPlayer->index != a  ){
			player_t* pAttacker = GET_PLAYER_POINTER_I(a);
			pPlayer->killer = pAttacker;
			pPlayer->custom = &pPlayer->pAPpl_stats[a];
			stat_t* aCustom = pAttacker->custom;
			pAttacker->custom = &pPlayer->aAPpl_stats[a];
			len += sprintf( &szBuffor[len], smv_putvars( sm_attackers->string )  );	
			if ( len ) {  // convert '\n' to "\n"
				szBuffor[len-1] = '\\';
				szBuffor[len] = 'n';
				szBuffor[++len] = 0;
			}
			pAttacker->custom = aCustom;
		}
	pPlayer->killer = pAttacker;
	pPlayer->custom = pCustom;
	return szBuffor;
}

/*
char* GetAttackersList(player_t *pPlayer)
{
	sm.player = pPlayer;
	player_t* pKiller = pPlayer->killer;
	stat_t* pCustom = pPlayer->custom;
	*szBuffor = 0;
	int len = 0;
	for (int a = 1; a <= gpGlobals->maxClients; ++a) {
		player_t* pAttacker = GET_PLAYER_POINTER_I(a);
		if ( pAttacker->pPpl_stats[pPlayer->index].hits && pPlayer->index != a ) {
			pPlayer->killer = pAttacker;
			pPlayer->custom = &pPlayer->pPpl_stats[a];
			stat_t* aCustom = pAttacker->custom;
			pAttacker->custom = &pAttacker->pPpl_stats[pPlayer->index];
			len += sprintf( &szBuffor[len], smv_putvars( sm_attackers->string )  );	
			if ( len ) {
				szBuffor[len-1] = '\\';
				szBuffor[len] = 'n';
				szBuffor[++len] = 0;
			}
			pAttacker->custom = aCustom;
		}
	}
	pPlayer->killer = pKiller;
	pPlayer->custom = pCustom;
	return szBuffor;
}
#endif */

// Fill vars with player stats and all weapon stats
char* MakeStats(player_t *pPlayer, char* pList)
{
	sm.player = pPlayer;
	int len = sprintf(pList,"%s",smv_putvars(sm_playerstats->string));
	for (int i = 0; i < MAX_WEAPONS ; ++i)	{
		if ( pPlayer->pWpn_stats[i].shots || pPlayer->pWpn_stats[i].kills)	{
			pPlayer->custom = &pPlayer->pWpn_stats[i];
			len += sprintf(&pList[len],"%s",smv_putvars(sm_weaponstats->string));
		}
	}
	return pList;
}

bool FloodProtection( float* time )
{
	if ( !sm_flood->value )	
		return true;

	if ( *time < gpGlobals->time )
	{
		*time = gpGlobals->time + sm_flood->value;
		return true;
	}

	return false;
}

unsigned short FixedUnsigned16( float value, float scale ){
	int output;
	output = int(value * scale);
	if ( output < 0 )
		output = 0;
	if ( output > 0xFFFF )
		output = 0xFFFF;
	return (unsigned short)output;
}

short FixedSigned16( float value, float scale ){
	int output;
	output = int(value * scale);
	if ( output > 32767 )
		output = 32767;
	if ( output < -32768 )
		output = -32768;
	return (short)output;
}

void UTIL_HudMessage(edict_t *pEntity, const hudtextparms_t &textparms, char *pMessage)
{
	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity );
		WRITE_BYTE(29); 
		WRITE_BYTE(textparms.channel & 0xFF); 
		WRITE_SHORT(FixedSigned16(textparms.x, (1<<13) ));
		WRITE_SHORT(FixedSigned16(textparms.y, (1<<13) ));
		WRITE_BYTE(textparms.effect);
		WRITE_BYTE(textparms.r1);
		WRITE_BYTE(textparms.g1);
		WRITE_BYTE(textparms.b1);
		WRITE_BYTE(0);
		WRITE_BYTE(255);
		WRITE_BYTE(255);
		WRITE_BYTE(250);
		WRITE_BYTE(0);
		WRITE_SHORT(FixedUnsigned16(textparms.fadeinTime, (1<<8) ));
		WRITE_SHORT(FixedUnsigned16(textparms.fadeoutTime, (1<<8) ));
		WRITE_SHORT(FixedUnsigned16(textparms.holdTime, (1<<8) ));
	if (textparms.effect==2) 
		WRITE_SHORT(FixedUnsigned16(textparms.fxTime, (1<<8) ) );
		WRITE_STRING(pMessage);
	MESSAGE_END();
}

void UTIL_HudMessage2(edict_t *pEntity, const hudtextparms_t &textparms, char *pMessage, int channel)
{
	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity );
		WRITE_BYTE(29); 
		WRITE_BYTE(channel & 0xFF); 
		WRITE_SHORT(FixedSigned16(textparms.x, (1<<13) ));
		WRITE_SHORT(FixedSigned16(textparms.y, (1<<13) ));
		WRITE_BYTE(textparms.effect);
		WRITE_BYTE(textparms.r1);
		WRITE_BYTE(textparms.g1);
		WRITE_BYTE(textparms.b1);
		WRITE_BYTE(0);
		WRITE_BYTE(255);
		WRITE_BYTE(255);
		WRITE_BYTE(250);
		WRITE_BYTE(0);
		WRITE_SHORT(FixedUnsigned16(textparms.fadeinTime, (1<<8) ));
		WRITE_SHORT(FixedUnsigned16(textparms.fadeoutTime, (1<<8) ));
		WRITE_SHORT(FixedUnsigned16(textparms.holdTime, (1<<8) ));
	if (textparms.effect==2) 
		WRITE_SHORT(FixedUnsigned16(textparms.fxTime, (1<<8) ) );
		WRITE_STRING(pMessage);
	MESSAGE_END();
}

int UTIL_NextChannel(player_t *pPlayer)
{

	pPlayer->next_channel = ++pPlayer->next_channel > 3 ? 1 : pPlayer->next_channel;
	return pPlayer->next_channel;
}

// used to check if argument specifing the player
bool UTIL_ValidPlayer(player_t* pPlayer, const char* arg){
	if ( !pPlayer->ingame || pPlayer->bot  )
		return false;
	if ( *arg == '*' && *(arg+1) == 0 )
		return true;
	if ( *arg == '#' && GETPLAYERUSERID( pPlayer->pEdict ) == atoi( arg+1 ) )
		return true;
	if ( !strcmp( pPlayer->team_name , arg ) )
		return true;
	return false;
}
